@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2019, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -35,6 +35,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <errno.h>
 #include <time.h>
 #include <string.h>
@@ -51,32 +52,28 @@
 #include <stdint.h>
 #endif
 
+#include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
+#include "flist.h"
 #include "nfnet.h"
 #include "bookkeeper.h"
 #include "collector.h"
 #include "exporter.h"
-#include "nf_common.h"
+#include "output_util.h"
+#include "output_raw.h"
+#include "output_pipe.h"
+#include "output_csv.h"
 #include "output_json.h"
 #include "netflow_v5_v7.h"
 #include "netflow_v9.h"
-#include "rbtree.h"
 #include "nftree.h"
 #include "nfprof.h"
-#include "nfdump.h"
 #include "nflowcache.h"
 #include "nfstat.h"
 #include "nfexport.h"
 #include "ipconv.h"
-#include "util.h"
-#include "flist.h"
-
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
 
 /* hash parameters */
 #define NumPrealloc 128000
@@ -84,7 +81,7 @@
 #define AGGR_SIZE 7
 
 /* Global Variables */
-FilterEngine_data_t	*Engine;
+FilterEngine_t	*Engine;
 
 extern char	*FilterFilename;
 extern uint32_t loopcnt;
@@ -93,7 +90,7 @@ extern uint32_t loopcnt;
 const char *nfdump_version = VERSION;
 
 static uint64_t total_bytes;
-static uint32_t total_flows;
+static uint32_t recordCount;
 static uint32_t skipped_blocks;
 static uint32_t	is_anonymized;
 static time_t 	t_first_flow, t_last_flow;
@@ -106,7 +103,7 @@ int hash_skip = 0;
 
 extension_map_list_t *extension_map_list;
 
-extern generic_exporter_t **exporter_list;
+extern exporter_t **exporter_list;
 /*
  * Output Formats:
  * User defined output formats can be compiled into nfdump, for easy access
@@ -210,28 +207,30 @@ extern generic_exporter_t **exporter_list;
  * 5. Recompile nfdump
  */
 
+static void flow_record_to_null(void *record, char ** s, int tag);
+
 // Assign print functions for all output options -o
 // Teminated with a NULL record
 printmap_t printmap[] = {
-	{ "raw",		format_file_block_record,  	NULL 			},
-	{ "line", 		format_special,      		FORMAT_line 	},
-	{ "long", 		format_special, 			FORMAT_long 	},
-	{ "extended",	format_special, 			FORMAT_extended	},
-	{ "biline", 	format_special,      		FORMAT_biline 	},
-	{ "bilong", 	format_special,      		FORMAT_bilong 	},
-	{ "pipe", 		flow_record_to_pipe,      	NULL 			},
-	{ "json", 		flow_record_to_json,      	NULL 			},
-	{ "csv", 		flow_record_to_csv,      	NULL 			},
-	{ "null", 		flow_record_to_null,      	NULL 			},
+	{ "raw",		flow_record_to_raw,  		raw_prolog, raw_epilog, NULL },
+	{ "line", 		format_special,      		text_prolog, text_epilog, FORMAT_line },
+	{ "long", 		format_special, 			text_prolog, text_epilog, FORMAT_long },
+	{ "extended",	format_special, 			text_prolog, text_epilog, FORMAT_extended },
+	{ "biline", 	format_special,      		text_prolog, text_epilog, FORMAT_biline	},
+	{ "bilong", 	format_special,      		text_prolog, text_epilog, FORMAT_bilong	},
+	{ "pipe", 		flow_record_to_pipe,      	pipe_prolog, pipe_epilog, NULL },
+	{ "json", 		flow_record_to_json,      	json_prolog, json_epilog, NULL },
+	{ "csv", 		flow_record_to_csv,      	csv_prolog,  csv_epilog,  NULL },
+	{ "null", 		flow_record_to_null,      	text_prolog, text_epilog, NULL },
 #ifdef NSEL
-	{ "nsel",		format_special, 			FORMAT_nsel		},
-	{ "nel",		format_special, 			FORMAT_nel		},
+	{ "nsel",		format_special, 			text_prolog, text_epilog, FORMAT_nsel },
+	{ "nel",		format_special, 			text_prolog, text_epilog, FORMAT_nel },
 #endif
 
 // add your formats here
 
 // This is always the last line
-	{ NULL,			NULL,                       NULL			}
+	{ NULL, NULL, NULL, NULL, "" }
 };
 
 // For automatic output format generation in case of custom aggregation
@@ -244,11 +243,11 @@ printmap_t printmap[] = {
 /* Function Prototypes */
 static void usage(char *name);
 
-static void PrintSummary(stat_record_t *stat_record, int plain_numbers, int csv_output);
+static void PrintSummary(stat_record_t *stat_record, outputParams_t *outputParams);
 
 static stat_record_t process_data(char *wfile, int element_stat, int flow_stat, int sort_flows,
-	printer_t print_header, printer_t print_record, time_t twin_start, time_t twin_end, 
-	uint64_t limitflows, int tag, int compress);
+	printer_t print_record, time_t twin_start, time_t twin_end, 
+	uint64_t limitRecords, outputParams_t *outputParams, int compress);
 
 /* Functions */
 
@@ -268,7 +267,7 @@ static void usage(char *name) {
 					"-w <file>\twrite output to file\n"
 					"-f\t\tread netflow filter from file\n"
 					"-n\t\tDefine number of top N for stat or sorted output.\n"
-					"-c\t\tLimit number of records to read from source(es)\n"
+					"-c\t\tLimit number of matching records\n"
 					"-D <dns>\tUse nameserver <dns> for host lookup.\n"
 					"-N\t\tPrint plain numbers\n"
 					"-s <expr>[/<order>]\tGenerate statistics for <expr> any valid record element.\n"
@@ -300,6 +299,7 @@ static void usage(char *name) {
 					"\t\t csv      ',' separated, machine parseable output format.\n"
 					"\t\t json     json output format.\n"
 					"\t\t pipe     '|' separated legacy machine parseable output format.\n"
+					"\t\t null     no flow records, but statistics output.\n"
 					"\t\t\tmode may be extended by '6' for full IPv6 listing. e.g.long6, extended6.\n"
 					"-E <file>\tPrint exporter and sampling info for collected flows.\n"
 					"-v <file>\tverify netflow data file. Print version and blocks.\n"
@@ -310,8 +310,11 @@ static void usage(char *name) {
 					"\t\tyyyy/MM/dd.hh:mm:ss[-yyyy/MM/dd.hh:mm:ss]\n", name);
 } /* usage */
 
+static void flow_record_to_null(void *record, char ** s, int tag) {
+	// empty - do not list any flows
+} // End of flow_record_to_null
 
-static void PrintSummary(stat_record_t *stat_record, int plain_numbers, int csv_output) {
+static void PrintSummary(stat_record_t *stat_record, outputParams_t *outputParams) {
 static double	duration;
 uint64_t	bps, pps, bpp;
 char 		byte_str[NUMBER_STRING_SIZE], packet_str[NUMBER_STRING_SIZE];
@@ -330,24 +333,19 @@ char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_
 		pps = stat_record->numpackets / duration;			// packets per second
 		bpp = stat_record->numpackets ? stat_record->numbytes / stat_record->numpackets : 0;    // Bytes per Packet
 	}
-	if ( csv_output ) {
+	if ( outputParams->modeCsv ) {
 		printf("Summary\n");
 		printf("flows,bytes,packets,avg_bps,avg_pps,avg_bpp\n");
 		printf("%llu,%llu,%llu,%llu,%llu,%llu\n",
 			(long long unsigned)stat_record->numflows, (long long unsigned)stat_record->numbytes, 
 			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
 			(long long unsigned)pps, (long long unsigned)bpp );
-	} else if ( plain_numbers ) {
-		printf("Summary: total flows: %llu, total bytes: %llu, total packets: %llu, avg bps: %llu, avg pps: %llu, avg bpp: %llu\n",
-			(long long unsigned)stat_record->numflows, (long long unsigned)stat_record->numbytes, 
-			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
-			(long long unsigned)pps, (long long unsigned)bpp );
 	} else {
-		format_number(stat_record->numbytes, byte_str, DO_SCALE_NUMBER, VAR_LENGTH);
-		format_number(stat_record->numpackets, packet_str, DO_SCALE_NUMBER, VAR_LENGTH);
-		format_number(bps, bps_str, DO_SCALE_NUMBER, VAR_LENGTH);
-		format_number(pps, pps_str, DO_SCALE_NUMBER, VAR_LENGTH);
-		format_number(bpp, bpp_str, DO_SCALE_NUMBER, VAR_LENGTH);
+		format_number(stat_record->numbytes, byte_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(stat_record->numpackets, packet_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(bps, bps_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(pps, pps_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(bpp, bpp_str, outputParams->printPlain, VAR_LENGTH);
 		printf("Summary: total flows: %llu, total bytes: %s, total packets: %s, avg bps: %s, avg pps: %s, avg bpp: %s\n",
 		(unsigned long long)stat_record->numflows, byte_str, packet_str, bps_str, pps_str, bpp_str );
 	}
@@ -355,8 +353,8 @@ char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_
 } // End of PrintSummary
 
 stat_record_t process_data(char *wfile, int element_stat, int flow_stat, int sort_flows,
-	printer_t print_header, printer_t print_record, time_t twin_start, time_t twin_end, 
-	uint64_t limitflows, int tag, int compress) {
+	printer_t print_record, time_t twin_start, time_t twin_end, 
+	uint64_t limitRecords, outputParams_t *outputParams, int compress) {
 common_record_t 	*flow_record, *record_ptr;
 master_record_t		*master_record;
 nffile_t			*nffile_w, *nffile_r;
@@ -456,15 +454,9 @@ int 				done, write_file;
 				total_bytes += ret;
 		}
 
-		if ( nffile_r->block_header->id == Large_BLOCK_Type ) {
-			// skip
-			printf("Xstat block skipped ...\n");
-			continue;
-		}
-
 		if ( nffile_r->block_header->id != DATA_BLOCK_TYPE_2 ) {
 			if ( nffile_r->block_header->id == DATA_BLOCK_TYPE_1 ) {
-				LogError("Can't process nfdump 1.5.x block type 1. Add --enable-compat15 to compile compatibility code. Skip block.\n");
+				LogError("nfdump 1.5.x block type 1 no longer supported. Skip block.\n");
 			} else {
 				LogError("Can't process block type %u. Skip block.\n", nffile_r->block_header->id);
 			}
@@ -474,7 +466,7 @@ int 				done, write_file;
 
 		uint32_t sumSize = 0;
 		record_ptr = nffile_r->buff_ptr;
-		for ( i=0; i < nffile_r->block_header->NumRecords; i++ ) {
+		for ( i=0; i < nffile_r->block_header->NumRecords && !done; i++ ) {
 			flow_record = record_ptr;
 			if ( (sumSize + record_ptr->size) > ret || (record_ptr->size < sizeof(record_header_t)) ) {
 				LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
@@ -488,7 +480,7 @@ int 				done, write_file;
 				case CommonRecordType: {
 					int match;
 					uint32_t map_id;
-					generic_exporter_t *exp_info;
+					exporter_t *exp_info;
 
 					// valid flow_record converted if needed
 					map_id = flow_record->ext_map;
@@ -504,7 +496,6 @@ int 				done, write_file;
 						continue;
 					} 
 
-					total_flows++;
 					master_record = &(extension_map_list->slot[map_id]->master_record);
 					Engine->nfrecord = (uint64_t *)master_record;
 					ExpandRecord_v2( flow_record, extension_map_list->slot[map_id], 
@@ -513,7 +504,7 @@ int 				done, write_file;
 					// Time based filter
 					// if no time filter is given, the result is always true
 					match  = twin_start && (master_record->first < twin_start || master_record->last > twin_end) ? 0 : 1;
-					match &= limitflows ? stat_record.numflows < limitflows : 1;
+					match &= limitRecords ? stat_record.numflows < limitRecords : 1;
 
 					// filter netflow record with user supplied filter
 					if ( match ) 
@@ -525,6 +516,7 @@ int 				done, write_file;
 						// go to next record
 						continue;
 					}
+					recordCount++;
 
 					// Records passed filter -> continue record processing
 					// Update statistics
@@ -553,13 +545,9 @@ int 				done, write_file;
 						} else if ( print_record ) {
 							char *string;
 							// if we need to print out this record
-							print_record(master_record, &string, tag);
+							print_record(master_record, &string, outputParams->doTag);
 							if ( string ) {
-								if ( limitflows ) {
-									if ( (stat_record.numflows <= limitflows) )
-										printf("%s\n", string);
-								} else 
-									printf("%s\n", string);
+								printf("%s\n", string);
 							}
 						} else { 
 							// mutually exclusive conditions should prevent executing this code
@@ -584,9 +572,9 @@ int 				done, write_file;
 							exit(255);
 					}
 					} break;
-				case ExporterRecordType:
-				case SamplerRecordype:
-						// Silently skip exporter records
+				case LegacyRecordType1:
+				case LegacyRecordType2:
+						// Silently skip legacy records
 					break;
 				case ExporterInfoRecordType: {
 					int ret = AddExporterInfo((exporter_info_record_t *)record_ptr);
@@ -614,15 +602,15 @@ int 				done, write_file;
 				}
 			}
 
-		// Advance pointer by number of bytes for netflow record
-		record_ptr = (common_record_t *)((pointer_addr_t)record_ptr + record_ptr->size);	
+			// Advance pointer by number of bytes for netflow record
+			record_ptr = (common_record_t *)((pointer_addr_t)record_ptr + record_ptr->size);	
 
+			// check if we are done, due to -c option 
+			if ( limitRecords ) 
+				done = recordCount >= limitRecords;
 
 		} // for all records
 
-		// check if we are done, due to -c option 
-		if ( limitflows ) 
-			done = stat_record.numflows >= limitflows;
 
 	} // while
 
@@ -657,17 +645,20 @@ int 				done, write_file;
 int main( int argc, char **argv ) {
 struct stat stat_buff;
 stat_record_t	sum_stat;
-printer_t 	print_header, print_record;
+outputParams_t *outputParams;
+printer_t	   print_record;
+func_prolog_t  print_prolog;
+func_epilog_t  print_epilog;
 nfprof_t 	profile_data;
 char 		*rfile, *Rfile, *Mdirs, *wfile, *ffile, *filter, *tstring, *stat_type;
-char		*byte_limit_string, *packet_limit_string, *print_format, *record_header;
+char		*byte_limit_string, *packet_limit_string, *print_format;
 char		*print_order, *query_file, *nameserver, *aggr_fmt;
 int 		c, ffd, ret, element_stat, fdump;
-int 		i, user_format, quiet, flow_stat, topN, aggregate, aggregate_mask, bidir;
-int 		print_stat, syntax_only, date_sorted, do_tag, compress;
-int			plain_numbers, GuessDir, pipe_output, csv_output, ModifyCompress;
+int 		i, flow_stat, aggregate, aggregate_mask, bidir;
+int 		print_stat, syntax_only, date_sorted, compress;
+int			printPlain, GuessDir, ModifyCompress;
 time_t 		t_start, t_end;
-uint32_t	limitflows;
+uint32_t	limitRecords;
 char 		Ident[IDENTLEN];
 
 	rfile = Rfile = Mdirs = wfile = ffile = filter = tstring = stat_type = NULL;
@@ -677,34 +668,35 @@ char 		Ident[IDENTLEN];
 	bidir			= 0;
 	t_start = t_end = 0;
 	syntax_only	    = 0;
-	topN	        = -1;
 	flow_stat       = 0;
 	print_stat      = 0;
 	element_stat  	= 0;
-	limitflows		= 0;
+	limitRecords	= 0;
 	date_sorted		= 0;
 	total_bytes		= 0;
-	total_flows		= 0;
+	recordCount		= 0;
 	skipped_blocks	= 0;
-	do_tag			= 0;
-	quiet			= 0;
-	user_format		= 0;
+	printPlain		= 0;
 	compress		= NOT_COMPRESSED;
-	plain_numbers   = 0;
-	pipe_output		= 0;
-	csv_output		= 0;
 	is_anonymized	= 0;
 	GuessDir		= 0;
 	nameserver		= NULL;
 
 	print_format    = NULL;
-	print_header 	= NULL;
 	print_record  	= NULL;
+	print_prolog	= NULL;
+	print_epilog	= NULL;
 	print_order  	= NULL;
 	query_file		= NULL;
 	ModifyCompress	= -1;
 	aggr_fmt		= NULL;
-	record_header 	= NULL;
+
+	outputParams	= calloc(1, sizeof(outputParams_t));
+	if ( !outputParams ) {
+		LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		exit(255);
+	}
+	outputParams->topN = -1;
 
 	Ident[0] = '\0';
 
@@ -754,7 +746,7 @@ char 		Ident[IDENTLEN];
 				syntax_only = 1;
 				break;
 			case 'q':
-				quiet = 1;
+				outputParams->quiet = true;
 				break;
 			case 'j':
 				if ( compress ) {
@@ -778,8 +770,8 @@ char 		Ident[IDENTLEN];
 				compress = LZO_COMPRESSED;
 				break;
 			case 'c':	
-				limitflows = atoi(optarg);
-				if ( !limitflows ) {
+				limitRecords = atoi(optarg);
+				if ( !limitRecords ) {
 					LogError("Option -c needs a number > 0\n");
 					exit(255);
 				}
@@ -811,7 +803,7 @@ char 		Ident[IDENTLEN];
 				byte_limit_string = optarg;
 				break;
 			case 'N':
-				plain_numbers = 1;
+				printPlain = 1;
 				break;
 			case 'f':
 				ffile = optarg;
@@ -838,6 +830,11 @@ char 		Ident[IDENTLEN];
 				break;
 			case 'o':	// output mode
 				print_format = optarg;
+				// limit input chars
+				if ( strlen(print_format) > 512 ) {
+					LogError("Length of ouput format string too big - > 512\n");
+					exit(255);
+				}
 				break;
 			case 'O': {	// stat order by
 				int ret;
@@ -856,18 +853,18 @@ char 		Ident[IDENTLEN];
 				wfile = optarg;
 				break;
 			case 'n':
-				topN = atoi(optarg);
-				if ( topN < 0 ) {
-					LogError("TopnN number %i out of range\n", topN);
+				outputParams->topN = atoi(optarg);
+				if ( outputParams->topN < 0 ) {
+					LogError("TopN number %i out of range\n", outputParams->topN);
 					exit(255);
 				}
 				break;
 			case 'T':
-				do_tag = 1;
+				outputParams->doTag = true;
 				break;
 			case 'i':
-				strncpy(Ident, optarg, IDENT_SIZE);
-				Ident[IDENT_SIZE - 1] = 0;
+				strncpy(Ident, optarg, IDENTLEN);
+				Ident[IDENTLEN - 1] = 0;
 				if ( strchr(Ident, ' ') ) {
 					LogError("Ident must not contain spaces\n");
 					exit(255);
@@ -924,11 +921,13 @@ char 		Ident[IDENTLEN];
 		exit(0);
 	}
 
-	if ( (element_stat || flow_stat) && (topN == -1)  ) 
-		topN = 10;
-
-	if ( topN < 0 )
-		topN = 0;
+	if ( outputParams->topN < 0 ) {
+		if ( flow_stat || element_stat ) {
+			outputParams->topN = 10;
+		} else {
+			outputParams->topN = 0;
+		}
+	}
 
 	if ( (element_stat && !flow_stat) && aggregate_mask ) {
 		LogError("Warning: Aggregation ignored for element statistics\n");
@@ -997,20 +996,14 @@ char 		Ident[IDENTLEN];
 			print_format = DefaultMode;
 	}
 
-	// limit input chars
-	if ( strlen(print_format) > 512 ) {
-		LogError("Length of ouput format string too big - > 512\n");
-		exit(255);
-	}
 	if ( strncasecmp(print_format, "fmt:", 4) == 0 ) {
 		// special user defined output format
 		char *format = &print_format[4];
 		if ( strlen(format) ) {
-			if ( !ParseOutputFormat(format, plain_numbers, printmap) )
+			if ( !ParseOutputFormat(format, printPlain, printmap) )
 				exit(255);
 			print_record  = format_special;
-			record_header = get_record_header();
-			user_format	  = 1;
+			print_prolog  = text_prolog;
 		} else {
 			LogError("Missing format description for user defined output format!\n");
 			exit(255);
@@ -1032,25 +1025,27 @@ char 		Ident[IDENTLEN];
 		while ( printmap[i].printmode ) {
 			if ( strncasecmp(print_format, printmap[i].printmode, MAXMODELEN) == 0 ) {
 				if ( printmap[i].Format ) {
-					if ( !ParseOutputFormat(printmap[i].Format, plain_numbers, printmap) )
+					if ( !ParseOutputFormat(printmap[i].Format, printPlain, printmap) )
 						exit(255);
 					// predefined custom format
-					print_record  = printmap[i].func;
-					record_header = get_record_header();
-					user_format	  = 1;
+					print_record  = printmap[i].func_record;
+					print_prolog  = printmap[i].func_prolog;
+					print_epilog  = printmap[i].func_epilog;
 				} else {
 					// To support the pipe output format for element stats - check for pipe, and remember this
 					if ( strncasecmp(print_format, "pipe", MAXMODELEN) == 0 ) {
-						pipe_output = 1;
+						outputParams->modePipe = true;
 					}
 					if ( strncasecmp(print_format, "csv", MAXMODELEN) == 0 ) {
-						csv_output = 1;
-						set_record_header();
-						record_header = get_record_header();
+						outputParams->modeCsv = true;
+					}
+					if ( strncasecmp(print_format, "json", MAXMODELEN) == 0 ) {
+						outputParams->modeJson = true;
 					}
 					// predefined static format
-					print_record  = printmap[i].func;
-					user_format	  = 0;
+					print_record  = printmap[i].func_record;
+					print_prolog  = printmap[i].func_prolog;
+					print_epilog  = printmap[i].func_epilog;
 				}
 				break;
 			}
@@ -1063,10 +1058,6 @@ char 		Ident[IDENTLEN];
 		exit(255);
 	}
 
-	// this is the only case, where headers are printed.
-	if ( strncasecmp(print_format, "raw", 16) == 0 )
-		print_header = format_file_block_header;
-	
 	if ( aggregate && (flow_stat || element_stat) ) {
 		aggregate = 0;
 		LogError("Command line switch -s overwrites -a\n");
@@ -1110,7 +1101,7 @@ char 		Ident[IDENTLEN];
 
 	if ( fdump ) {
 		printf("StartNode: %i Engine: %s\n", Engine->StartNode, Engine->Extended ? "Extended" : "Fast");
-		DumpList(Engine);
+		DumpEngine(Engine);
 		exit(0);
 	}
 
@@ -1136,24 +1127,16 @@ char 		Ident[IDENTLEN];
 	}
 
 
-	if ( !(flow_stat || element_stat || wfile || quiet ) && record_header ) {
-		if ( user_format ) {
-			printf("%s\n", record_header);
-		} else {
-			// static format - no static format with header any more, but keep code anyway
-			if ( Getv6Mode() ) {
-				printf("%s\n", record_header);
-			} else
-				printf("%s\n", record_header);
-		}
+	if ( !(flow_stat || element_stat || wfile || outputParams->quiet ) && print_prolog ) {
+		print_prolog();
 	}
 
 	nfprof_start(&profile_data);
 	sum_stat = process_data(wfile, element_stat, aggregate || flow_stat, print_order != NULL,
-						print_header, print_record, t_start, t_end, 
-						limitflows, do_tag, compress);
-	nfprof_end(&profile_data, total_flows);
-
+						print_record, t_start, t_end, 
+						limitRecords, outputParams, compress);
+	nfprof_end(&profile_data, recordCount);
+	
 	if ( total_bytes == 0 ) {
 		printf("No matched flows\n");
 		exit(0);
@@ -1164,7 +1147,7 @@ char 		Ident[IDENTLEN];
 			nffile_t *nffile = OpenNewFile(wfile, NULL, compress, is_anonymized, NULL);
 			if ( !nffile ) 
 				exit(255);
-			if ( ExportFlowTable(nffile, aggregate, bidir, date_sorted, extension_map_list) ) {
+			if ( ExportFlowTable(nffile, aggregate, bidir, GuessDir, date_sorted, extension_map_list) ) {
 				CloseUpdateFile(nffile, Ident );	
 			} else {
 				CloseFile(nffile);
@@ -1172,28 +1155,31 @@ char 		Ident[IDENTLEN];
 			}
 			DisposeFile(nffile);
 		} else {
-			PrintFlowTable(print_record, topN, do_tag, GuessDir, extension_map_list);
+			PrintFlowTable(print_record, outputParams, GuessDir, extension_map_list);
 		}
 	}
 
 	if (flow_stat) {
-		PrintFlowStat(record_header, print_record, topN, do_tag, quiet, csv_output, extension_map_list);
+		PrintFlowStat(print_prolog, print_record, outputParams, extension_map_list);
 #ifdef DEVEL
 		printf("Loopcnt: %u\n", loopcnt);
 #endif
 	} 
 
 	if (element_stat) {
-		PrintElementStat(&sum_stat, plain_numbers, record_header, print_record, topN, do_tag, quiet, pipe_output, csv_output);
+		PrintElementStat(&sum_stat, outputParams, print_record);
 	} 
 
-	if ( !quiet ) {
-		if ( csv_output ) {
-			PrintSummary(&sum_stat, plain_numbers, csv_output);
+	if ( print_epilog ) {
+		print_epilog();
+	}
+	if ( !outputParams->quiet && !outputParams->modeJson ) {
+		if ( outputParams->modeCsv ) {
+			PrintSummary(&sum_stat, outputParams);
 		} else if ( !wfile ) {
 			if (is_anonymized)
 				printf("IP addresses anonymised\n");
-			PrintSummary(&sum_stat, plain_numbers, csv_output);
+			PrintSummary(&sum_stat, outputParams);
 			if ( t_last_flow == 0 ) {
 				// in case of a pre 1.6.6 collected and empty flow file
  				printf("Time window: <unknown>\n");
@@ -1201,7 +1187,7 @@ char 		Ident[IDENTLEN];
  				printf("Time window: %s\n", TimeString(t_first_flow, t_last_flow));
 			}
 			printf("Total flows processed: %u, Blocks skipped: %u, Bytes read: %llu\n", 
-				total_flows, skipped_blocks, (unsigned long long)total_bytes);
+				recordCount, skipped_blocks, (unsigned long long)total_bytes);
 			nfprof_print(&profile_data, stdout);
 		}
 	}
