@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2019, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -45,15 +45,9 @@
 #include <stdint.h>
 #endif
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
-#include "nf_common.h"
-#include "nffile.h"
 #include "util.h"
+#include "nfdump.h"
+#include "nffile.h"
 #include "nfx.h"
 
 /* global vars */
@@ -261,6 +255,7 @@ uint16_t map_id;
 		}
 		l->ref_count 	= 0;
 		l->next 		= NULL;
+		l->exportMap 	= NULL;
 		memset((void *)&l->master_record, 0, sizeof(master_record_t));
 
 		l->map   = (extension_map_t *)malloc((ssize_t)map->size);
