@@ -1,9 +1,5 @@
 /*
- *  Copyright (c) 2018, Peter Haag
- *  Copyright (c) 2017, Peter Haag
- *  Copyright (c) 2016, Peter Haag
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -70,12 +66,13 @@
 #endif
 
 #include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
-#include "nf_common.h"
 #include "nfnet.h"
 #include "bookkeeper.h"
 #include "collector.h"
+#include "launch.h"
 #include "flist.h"
 #include "nfstatfile.h"
 
@@ -97,7 +94,7 @@
 #define DEFAULTSFLOWPORT "6343"
 
 /* Global Variables */
-caddr_t		shmem;
+void *shmem;
 
 /* globals */
 int verbose = 0;
@@ -144,6 +141,7 @@ static void usage(char *name) {
 					"-I Ident\tset the ident string for stat file. (default 'none')\n"
 					"-H Add port histogram data to flow file.(default 'no')\n"
 					"-n Ident,IP,logdir\tAdd this flow source - multiple streams\n" 
+					"-N sourceFile\tAdd flows from sourceFile\n" 
 					"-P pidfile\tset the PID file\n"
 					"-R IP[/port]\tRepeat incoming packets to IP address/port. Max 8 repeaters\n"
 					"-x process\tlaunch process after a new file becomes available\n"
@@ -330,6 +328,19 @@ int		err;
 
 } // End of SetPriv
 
+static void format_file_block_header(data_block_header_t *header) {
+	
+	printf("\n"
+"File Block Header: \n"
+"  NumBlocks     =  %10u\n"
+"  Size          =  %10u\n"
+"  id         	 =  %10u\n",
+		header->NumRecords,
+		header->size,
+		header->id);
+
+} // End of format_file_block_header
+
 #include "nffile_inline.c"
 #include "collector_inline.c"
 
@@ -344,10 +355,9 @@ uint32_t	blast_cnt, blast_failures, ignored_packets;
 ssize_t		cnt;
 void 		*in_buff;
 int 		err;
-char 		*string;
 srecord_t	*commbuff;
 
-	Init_sflow();
+	Init_sflow(verbose);
 
 	in_buff  = malloc(NETWORK_INPUT_BUFF_SIZE);
 	if ( !in_buff ) {
@@ -357,7 +367,6 @@ srecord_t	*commbuff;
 
 	// init vars
 	commbuff = (srecord_t *)shmem;
-	string = NULL;
 
 	// Init each netflow source output data buffer
 	fs = FlowSource;
@@ -384,8 +393,8 @@ srecord_t	*commbuff;
 	cnt = 0;
 	ignored_packets  = 0;
 
-	// wake up at least at next time slot (twin) + some Overdue time
-	alarm(t_start + twin + OVERDUE_TIME - time(NULL));
+	// wake up at least at next time slot (twin) + 1s
+	alarm(t_start + twin + 1 - time(NULL));
 	/*
 	 * Main processing loop:
 	 * this loop, continues until done = 1, set by the signal handler
@@ -434,12 +443,12 @@ srecord_t	*commbuff;
 		t_now = tv.tv_sec;
 
 		if ( ((t_now - t_start) >= twin) || done ) {
-			char subfilename[64];
 			struct  tm *now;
-			char	*subdir, fmt[64];
+			char	*subdir, fmt[MAXTIMESTRING];
+
 			alarm(0);
 			now = localtime(&t_start);
-			strftime(fmt, sizeof fmt, time_extension, now);
+			strftime(fmt, sizeof(fmt), time_extension, now);
 
 			// prepare sub dir hierarchy
 			if ( use_subdirs ) {
@@ -450,15 +459,10 @@ srecord_t	*commbuff;
 			
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(subfilename, 63, "nfcapd.%s", fmt);
-				} else {
-					snprintf(subfilename, 63, "%s/nfcapd.%s", subdir, fmt);
 				}
 			} else {
 				subdir = NULL;
-				snprintf(subfilename, 63, "nfcapd.%s", fmt);
 			}
-			subfilename[63] = '\0';
 
 			// for each flow source update the stats, close the file and re-initialize the new file
 			fs = FlowSource;
@@ -469,23 +473,39 @@ srecord_t	*commbuff;
 
 				if ( verbose ) {
 					// Dump to stdout
-					format_file_block_header(nffile->block_header, &string, 0);
-					printf("%s\n", string);
+					format_file_block_header(nffile->block_header);
 				}
 
 				if ( nffile->block_header->NumRecords ) {
 					// flush current buffer to disc
 					if ( WriteBlock(nffile) <= 0 )
-						LogError("Ident: %s, failed to write output buffer to disk: '%s'" , fs->Ident, strerror(errno));
+						LogError("Ident: %s, failed to write output buffer to disk: '%s'" ,
+							fs->Ident, strerror(errno));
 				} // else - no new records in current block
 
 	
 				// prepare filename
-				snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/%s", fs->datadir, subfilename);
+				if ( subdir ) {
+					if ( SetupSubDir(fs->datadir, subdir, error, 255) ) {
+						snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/%s/nfcapd.%s", fs->datadir, subdir, fmt);
+					} else {
+						LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
+						// skip subdir - put flows directly into current directory
+						snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/nfcapd.%s", fs->datadir, fmt);
+					}
+				} else {
+					snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/nfcapd.%s", fs->datadir, fmt);
+				}
 				nfcapd_filename[MAXPATHLEN-1] = '\0';
-	
+
 				// update stat record
-				nffile->stat_record->first_seen 	= fs->first_seen/1000;
+				// if no flows were collected, fs->last_seen is still 0
+				// set first_seen to start of this time slot, with twin window size.
+				if ( fs->last_seen == 0 ) {
+					fs->first_seen = (uint64_t)1000 * (uint64_t)t_start;
+					fs->last_seen  = (uint64_t)1000 * (uint64_t)(t_start + twin);
+				}
+				nffile->stat_record->first_seen = fs->first_seen/1000;
 				nffile->stat_record->msec_first	= fs->first_seen - nffile->stat_record->first_seen*1000;
 				nffile->stat_record->last_seen 	= fs->last_seen/1000;
 				nffile->stat_record->msec_last	= fs->last_seen - nffile->stat_record->last_seen*1000;
@@ -495,12 +515,6 @@ srecord_t	*commbuff;
 				// Write Stat record and close file
 				CloseUpdateFile(nffile, fs->Ident);
 
-				if ( subdir && !SetupSubDir(fs->datadir, subdir, error, 255) ) {
-					// in this case the flows get lost! - the rename will fail
-					// but this should not happen anyway, unless i/o problems, inode problems etc.
-					LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
-				}
-
 				// if rename fails, we are in big trouble, as we need to get rid of the old .current file
 				// otherwise, we will loose flows and can not continue collecting new flows
 				err = rename(fs->current, nfcapd_filename);
@@ -524,7 +538,8 @@ srecord_t	*commbuff;
 
 				// log stats
 				LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
-					fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets, 
+					fs->Ident, (unsigned long long)nffile->stat_record->numflows,
+					(unsigned long long)nffile->stat_record->numpackets, 
 					(unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets);
 
 				// reset stat record
@@ -551,17 +566,16 @@ srecord_t	*commbuff;
 			if ( launcher_pid ) {
 				// Signal launcher
 		
-				// prepare filename for %f expansion
-				strncpy(commbuff->fname, subfilename, FNAME_SIZE-1);
-				commbuff->fname[FNAME_SIZE-1] = 0;
-				snprintf(commbuff->tstring, 16, "%i%02i%02i%02i%02i", 
-					now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
-				commbuff->tstring[15] = 0;
+				strncpy(commbuff->tstring, fmt, MAXTIMESTRING);
+				commbuff->tstring[MAXTIMESTRING-1] = '\0';
+
 				commbuff->tstamp = t_start;
-				if ( subdir ) 
-					strncpy(commbuff->subdir, subdir, FNAME_SIZE);
-				else
-					commbuff->subdir[0] = '\0';
+				if ( subdir ) {
+					snprintf(commbuff->fname, MAXPATHLEN-1, "%s/nfcapd.%s", subdir, fmt);
+				} else {
+					snprintf(commbuff->fname, MAXPATHLEN-1, "nfcapd.%s", fmt);
+				}
+				commbuff->fname[MAXPATHLEN-1] = '\0';
 
 				if ( launcher_alive ) {
 					LogInfo("Signal launcher");
@@ -581,10 +595,10 @@ srecord_t	*commbuff;
 			t_start += twin;
 			/* t_start = filename time stamp: begin of slot
 		 	* + twin = end of next time interval
-		 	* + OVERDUE_TIME = if no data is collected, this is at latest to act
+		 	* + 1 = if no data is collected, this is at latest to act
 		 	* - t_now = difference value to now
 		 	*/
-			alarm(t_start + twin + OVERDUE_TIME - t_now);
+			alarm(t_start + twin + 1 - t_now);
 		}
 
 		/* check for error condition or done . errno may only be EINTR */
@@ -660,7 +674,7 @@ int main(int argc, char **argv) {
  
 char	*bindhost, *datadir, pidstr[32], *launch_process;
 char	*userid, *groupid, *checkptr, *listenport, *mcastgroup, *extension_tags;
-char	*Ident, *pcap_file, *time_extension, pidfile[MAXPATHLEN];
+char	*Ident, *time_extension, pidfile[MAXPATHLEN];
 struct stat fstat;
 packet_function_t receive_packet;
 repeater_t repeater[MAX_REPEATERS];
@@ -671,6 +685,9 @@ time_t 	twin, t_start;
 int		sock, synctime, do_daemonize, expire, spec_time_extension, report_sequence;
 int		subdir_index, compress;
 int		c, i;
+#ifdef PCAP
+char	*pcap_file = NULL;
+#endif
 
 	receive_packet 	= recvfrom;
 	verbose = synctime = do_daemonize = 0;
@@ -689,8 +706,8 @@ int		c, i;
 	datadir	 		= NULL;
 	subdir_index	= 0;
 	time_extension	= "%Y%m%d%H%M";
-	expire			= 0;
 	spec_time_extension = 0;
+	expire			= 0;
 	compress		= NOT_COMPRESSED;
 	memset((void *)&repeater, 0, sizeof(repeater));
 	for ( i = 0; i < MAX_REPEATERS; i++ ) {
@@ -699,9 +716,8 @@ int		c, i;
 	Ident			= "none";
 	FlowSource		= NULL;
 	extension_tags	= DefaultExtensions;
-	pcap_file		= NULL;
 
-	while ((c = getopt(argc, argv, "46ewhEVI:DB:b:f:jl:n:p:J:P:R:S:T:t:x:ru:g:zZ")) != EOF) {
+	while ((c = getopt(argc, argv, "46ewhEVI:DB:b:f:jl:N:n:p:J:P:R:S:T:t:x:ru:g:zZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -746,6 +762,10 @@ int		c, i;
 				if ( AddFlowSource(&FlowSource, optarg) != 1 ) 
 					exit(255);
 				break;
+			case 'N':
+				if ( AddFlowSourceFromFile(&FlowSource, optarg) )
+					exit(255);
+				break;
 			case 'w':
 				synctime = 1;
 				break;
@@ -795,7 +815,12 @@ int		c, i;
 						exit(255);
 					}
 					tmp[MAXPATHLEN-1] = 0;
-					snprintf(pidfile, MAXPATHLEN - 1 - strlen(tmp), "%s/%s", tmp, optarg);
+					if ( (strlen(tmp) + strlen(optarg) + 3) < MAXPATHLEN ) {
+						snprintf(pidfile, MAXPATHLEN - 3 - strlen(tmp), "%s/%s", tmp, optarg);
+					} else {
+						fprintf(stderr, "pidfile MAXPATHLEN error:\n");
+						exit(255);
+					}
 				}
 				// pidfile now absolute path
 				pidfile[MAXPATHLEN-1] = 0;
@@ -848,13 +873,12 @@ int		c, i;
 				break;
 			case 't':
 				twin = atoi(optarg);
-				if ( twin <= 0 ) {
-					fprintf(stderr, "ERROR: time frame <= 0\n");
+				if ( twin < 2 ) {
+					LogError("time interval <= 2s not allowed");
 					exit(255);
 				}
 				if (twin < 60) {
-					fprintf(stderr, "WARNING, Very small time frame - < 60s!\n");
-					exit(255);
+					time_extension	= "%Y%m%d%H%M%S";
 				}
 				break;
 			case 'x':
@@ -907,7 +931,7 @@ int		c, i;
 		exit(255);
 	}
 
-	if ( !InitLog(argv[0], SYSLOG_FACILITY)) {
+	if ( !InitLog(do_daemonize, argv[0], SYSLOG_FACILITY, verbose) ) {
 		exit(255);
 	}
 
@@ -1024,7 +1048,7 @@ int		c, i;
 		// as well as shared memory
 		// prepare shared memory
 		shmem = mmap(0, sizeof(srecord_t), PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
-		if ( shmem == (caddr_t)-1 ) {
+		if ( shmem == MAP_FAILED ) {
 			LogError("mmap() error: %s", strerror(errno));
 			close(sock);
 			exit(255);
@@ -1035,7 +1059,7 @@ int		c, i;
 			case 0:
 				// child
 				close(sock);
-				launcher((char *)shmem, FlowSource, launch_process, expire);
+				launcher(shmem, FlowSource, launch_process, expire);
 				exit(0);
 				break;
 			case -1:
